@@ -5,7 +5,7 @@ use strict;
 use warnings;
 
 use Moo;
-use Types::Standard qw/Str Object Bool StrictNum/;
+use Types::Standard qw/Str Object Bool StrictNum Int HashRef ArrayRef/;
 use File::Copy;
 use File::Spec;
 use File::Temp;
@@ -27,17 +27,18 @@ PDF::Cropmarks - Add cropmarks to existing PDFs
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 SYNOPSIS
 
 This module prepares PDF for printing adding the cropmarks, usually on
 a larger physical page, doing the same thing the LaTeX package "crop"
-does.
+does. It also takes care of the paper thickness, shifting the logical
+pages to compensate the folding.
 
 It comes with a ready-made script, C<pdf-cropmarks.pl>. E.g.
 
@@ -135,6 +136,23 @@ Default: 8pt
 The font size of the headers and footers with the job name, date, and
 page numbers.
 
+=head2 signature
+
+Default to 0, meaning that no signature is needed. If set to 1, means
+that all the pages should fit in a single signature, otherwise it
+should be a multiple of 4.
+
+=head2 paper_thickness
+
+When passing the signature option, the logical pages are shifted on
+the x axys by this amount to compensate the paper folding. Accept a
+measure.
+
+This option is active only when the signature is set (default to
+false) and twoside is true (the default). Default to 0.1mm, which is
+appropriate for the common paper 80g/m2. You can do the math measuring
+a stack height and dividing by the number of sheets.
+
 =cut
 
 has cropmark_length => (is => 'ro', isa => Str, default => sub { '12mm' });
@@ -147,6 +165,15 @@ has cropmark_length_in_pt => (is => 'lazy', isa => StrictNum);
 has cropmark_offset_in_pt => (is => 'lazy', isa => StrictNum);
 has font_size_in_pt => (is => 'lazy', isa => StrictNum);
 
+has signature => (is => 'rwp', isa => Int, default => sub { 0 });
+has paper_thickness => (is => 'ro', isa => Str, default => sub { '0.1mm' });
+has paper_thickness_in_pt => (is => 'lazy', isa => StrictNum);
+
+sub _build_paper_thickness_in_pt {
+    my $self = shift;
+    return $self->_string_to_pt($self->paper_thickness);
+}
+
 sub _build_cropmark_length_in_pt {
     my $self = shift;
     return $self->_string_to_pt($self->cropmark_length);
@@ -161,9 +188,96 @@ sub _build_font_size_in_pt {
     return $self->_string_to_pt($self->font_size);
 }
 
+has thickness_page_offsets => (is => 'lazy', isa => HashRef[HashRef]);
+
+sub _build_thickness_page_offsets {
+    my $self = shift;
+    my $total_pages = $self->total_output_pages;
+    my %out = map { $_ => 0 } (1 .. $total_pages);
+    if (my $signature = $self->signature) {
+        # convert to the real signature
+        if ($signature == 1) {
+            $signature = $total_pages;
+        }
+        die "Should have already died, signature not a multiple of four" if $signature % 4;
+        my $half = $signature / 2;
+        my $offset = $self->paper_thickness_in_pt * ($half / 2);
+        my $original_offset = $self->paper_thickness_in_pt * ($half / 2);
+        my $signature_number = 0;
+        foreach my $page (1 .. $total_pages) {
+            my $page_in_sig = $page % $signature || $signature;
+            if ($page_in_sig == 1) {
+                $offset = $original_offset;
+                $signature_number++;
+            }
+            print "page in sig / $signature_number : $page_in_sig\n" if DEBUG;
+            # odd pages triggers a stepping
+            if ($page_in_sig % 2) {
+                if ($page_in_sig > ($half + 1)) {
+                    $offset += $self->paper_thickness_in_pt;
+                }
+                elsif ($page_in_sig < $half) {
+                    $offset -= $self->paper_thickness_in_pt;
+                }
+            }
+            my $rounded = $self->_round($offset);
+            print "offset for page is $rounded\n" if DEBUG;
+            $out{$page} = {
+                           offset => $rounded,
+                           signature => $signature_number,
+                           signature_page => $page_in_sig,
+                          };
+        }
+    }
+    return \%out;
+}
+
+has total_input_pages => (is => 'lazy', isa => Int);
+
+sub _build_total_input_pages {
+    my $self = shift;
+    my $count = 0;
+    while ($self->in_pdf_object->openpage($count + 1)) {
+        $count++;
+    }
+    return $count;
+}
+
+has total_output_pages => (is => 'lazy', isa => Int);
+
+sub _build_total_output_pages {
+    my $self = shift;
+    my $total_input_pages = $self->total_input_pages;
+
+    if (my $signature = $self->signature) {
+        if ($signature == 1) {
+            # all the pages on a single signature
+            # round to the next multiple of 4
+            my $missing = 0;
+            if (my $modulo = $total_input_pages % 4) {
+                $missing = 4 - $modulo;
+            }
+            return $total_input_pages + $missing;
+        }
+        elsif ($signature % 4) {
+            die "Signature must be 1 or a multiple of 4\n";
+        }
+        else {
+            my $missing = 0;
+            if (my $modulo = $total_input_pages % $signature) {
+                $missing = $signature - $modulo;
+            }
+            return $total_input_pages + $missing;
+        }
+    }
+    else {
+        return $total_input_pages;
+    }
+}
+
 
 sub _measure_re {
-    return qr{([0-9]+(\.[0-9]+)?)
+    return qr{([0-9]+(\.[0-9]+)?)\s*
               (mm|in|pt|cm)}sxi;
 }
 
@@ -171,7 +285,7 @@ sub _string_to_pt {
     my ($self, $string) = @_;
     my %compute = (
                    mm => sub { $_[0] / (25.4 / 72) },
-                   in => sub { $_[0] / (1 /72) },
+                   in => sub { $_[0] / (1 / 72) },
                    pt => sub { $_[0] / 1 },
                    cm => sub { $_[0] / (25.4 / 72) * 10 },
                   );
@@ -179,7 +293,7 @@ sub _string_to_pt {
     if ($string =~ $re) {
         my $size = $1;
         my $unit = lc($3);
-        return sprintf('%.2f', $compute{$unit}->($size));
+        return $self->_round($compute{$unit}->($size));
     }
     else {
         die "Unparsable measure string $string";
@@ -231,7 +345,7 @@ has bottom => (is => 'ro', isa => Bool, default => sub { 1 });
 has inner => (is => 'ro', isa => Bool, default => sub { 1 });
 has outer => (is => 'ro', isa => Bool, default => sub { 1 });
 has twoside => (is => 'ro', isa => Bool, default => sub { 1 });
-
+has _is_closed => (is => 'rw', isa => Bool, default => sub { 0 });
 sub _build_in_pdf {
     my $self = shift;
     my $name = File::Spec->catfile($self->_tmpdir, 'in.pdf');
@@ -309,36 +423,46 @@ sub _paper_dimensions {
     }
 }
 
-
 sub add_cropmarks {
     my $self = shift;
+    die "add_cropmarks already called!" if $self->_is_closed;
     my $page = 1;
-    while (my $pageobj = $self->in_pdf_object->openpage($page)) {
+    foreach my $page (1 .. $self->total_output_pages) {
         print "Importing page $page\n" if DEBUG;
-        $self->_import_page($pageobj, $page);
-        $page++;
+        $self->_import_page($page);
     }
     $self->out_pdf_object->saveas($self->out_pdf);
     $self->in_pdf_object->end;
     $self->out_pdf_object->end;
     move($self->out_pdf, $self->output)
       or die "Cannot copy " . $self->out_pdf . ' to ' . $self->output;
+    $self->_is_closed(1);
     return $page;
 }
 
 sub _round {
     my ($self, $float) = @_;
     $float || 0;
-    return sprintf('%.2f', $float);
+    return sprintf('%.3f', $float);
+}
+
+has output_dimensions => (is => 'lazy', isa => ArrayRef);
+
+sub _build_output_dimensions {
+    my $self = shift;
+    # get the first page
+    my $in_page = $self->in_pdf_object->openpage(1);
+    return [ $in_page->get_mediabox ];
 }
 
 sub _import_page {
-    my ($self, $in_page, $page_number) = @_;
+    my ($self, $page_number) = @_;
+    my $in_page = $self->in_pdf_object->openpage($page_number);
     my $page = $self->out_pdf_object->page;
     my ($llx, $lly, $urx, $ury) = $page->get_mediabox;
     die "mediabox origins for output pdf should be zero" if $llx + $lly;
     print "$llx, $lly, $urx, $ury\n" if DEBUG;
-    my ($inllx, $inlly, $inurx, $inury) = $in_page->get_mediabox;
+    my ($inllx, $inlly, $inurx, $inury) = ($in_page ? $in_page->get_mediabox : @{$self->output_dimensions});
     print "$inllx, $inlly, $inurx, $inury\n" if DEBUG;
     die "mediabox origins for input pdf should be zero" if $inllx + $inlly;
     # place the content into page
@@ -377,12 +501,31 @@ sub _import_page {
             $offset_x *= 2;
         }
     }
-    print "Offsets are $offset_x, $offset_y\n" if DEBUG;
 
-    my $xo = $self->out_pdf_object->importPageIntoForm($self->in_pdf_object,
-                                                       $page_number);
-    my $gfx = $page->gfx;
-    $gfx->formimage($xo, $offset_x, $offset_y);
+    my $signature_mark = '';
+    if ($self->signature && $self->twoside) {
+        my $spec = $self->thickness_page_offsets->{$page_number};
+        my $paper_thickness = $spec->{offset};
+        die "$page_number not defined in " . Dumper($self->thickness_page_offsets)
+          unless defined $paper_thickness;
+        # recto pages, increase
+        if ($page_number % 2) {
+            $offset_x += $paper_thickness;
+        }
+        # verso pages, decrease
+        else {
+            $offset_x -= $paper_thickness;
+        }
+        $signature_mark = ' #' . $spec->{signature} . '/' . $spec->{signature_page};
+    }
+
+    print "Offsets are $offset_x, $offset_y\n" if DEBUG;
+    if ($in_page) {
+        my $xo = $self->out_pdf_object->importPageIntoForm($self->in_pdf_object,
+                                                           $page_number);
+        my $gfx = $page->gfx;
+        $gfx->formimage($xo, $offset_x, $offset_y);
+    }
     if (DEBUG) {
         my $line = $page->gfx;
         $line->strokecolor('black');
@@ -473,7 +616,7 @@ sub _import_page {
     $text->text($marker);
 
     my $text_marker = $self->basename . ' ' . $self->timestamp .
-      ' page ' . $page_number;
+      ' page ' . $page_number . $signature_mark;
     # and at the top and and the bottom add jobname + timestamp
     $text->translate(($inurx / 2) + $offset_x,
                      $offset_y + $inury + $crop_width);
@@ -496,6 +639,14 @@ sub _draw_line {
     $gfx->line($to_x, $to_y + $radius);
 }
 
+sub DESTROY {
+    my $self = shift;
+    unless ($self->_is_closed) {
+        $self->in_pdf_object->end;
+        $self->out_pdf_object->end;
+    }
+}
+
 =head1 AUTHOR
 
 Marco Pessotto, C<< <melmothx at gmail.com> >>
